; ------------------------------------------------------------------------------
;
;					X-Launcher - The Universal Launcher!
;
; ------------------------------------------------------------------------------
;
; AutoIt Version:			3.2.0.1
; Platform:					Windows
; Script Name:				X-Launcher
; Script Function:			Runs every application from a removable drive
; Script Version:			1.1.0.0
; Author:					Gabriele Tittonel <tittoproject@gmail.com>
; Home Page:				http://x-launcher.sourceforge.net/
; License:					GNU General Public License
;
; ------------------------------------------------------------------------------
#NoTrayIcon
AutoItSetOption('ExpandEnvStrings', 1)
AutoItSetOption('ExpandVarStrings', 1)

; ------------------------------------------------------------------------------
; ScriptIni
; ------------------------------------------------------------------------------

; Get script name from file name
$ScriptName = _FileInfo (@ScriptFullPath, 2)

; Get enviromental variable "XCONFIG"
If EnvGet('XCONFIG') <> '' Then
	$ScriptIni = EnvGet('XCONFIG')
Else
	$ScriptIni = @ScriptDir & '\' & $ScriptName & '.ini'
EndIf

; Get commandline option "--x-launcher-config"
If $CmdLine[0] <> 0 And StringInStr($CmdLine[1], '--x-launcher-config') Then
	$Cmd1 = 2
	$_xcmd = StringSplit($CmdLine[1], '=')
	Select
		Case $_xcmd[0] = 2
			$ScriptIni = _FullPath ($_xcmd[2])
		Case Else
			FileInstall('x-launcher.ini', $ScriptIni)
			Exit
	EndSelect
Else
	$Cmd1 = 1
EndIf

; Install ScriptIni if doesn't exists
$ScriptIniDir = _FileInfo ($ScriptIni, 0)

; Warning messages
$Lang = EnvGet('LANG')
Select
	Case $Lang = 'it'
		$NoIni = 'Il file di configurazione "' & $ScriptIni & '" non esiste.' & @CRLF & @CRLF & 'Ne verr creato uno con le opzioni predefinite.'
		$PleaseCheck = 'Si prega di controllare le impostazioni prima di proseguire.'
	Case Else
		$NoIni = 'The configuration file "' & $ScriptIni & '" does not exist.' & @CRLF & @CRLF & 'One with the default options will be created.'
		$PleaseCheck = 'Please check the settings before continue.'
EndSelect

; Install configuration file
If Not FileExists($ScriptIni) Then
	MsgBox(48, $ScriptName, $NoIni)
	MsgBox(48, $ScriptName, $PleaseCheck)
	DirCreate($ScriptIniDir)
	FileInstall('x-launcher.ini', $ScriptIni)
	FileChangeDir($ScriptIniDir)
	RunWait(@ComSpec & ' /c "' & $ScriptIni & '"', '', @SW_HIDE)
	Exit
EndIf

; ------------------------------------------------------------------------------
; Script Setup
; ------------------------------------------------------------------------------
$AppName = _IniReadPlus($ScriptIni, 'Setup', 'AppName', 'MyApp')
$AppVer = _IniReadPlus($ScriptIni, 'Setup', 'AppVer', '')
$Profile = _IniReadPlus($ScriptIni, 'Setup', 'Profile', 'Default')

; ------------------------------------------------------------------------------
; Global Preferences
; ------------------------------------------------------------------------------
$GlobalConfig = @ScriptDir & '\X-Launcher.cfg'
If Not FileExists($GlobalConfig) Then $GlobalConfig = $ScriptIni

; Setup
$UserName 	= _IniReadPlus($GlobalConfig, 'Setup', 'UserName', 'User')
$Lang 		= _IniReadPlus($GlobalConfig, 'Setup', 'Lang', $Lang)

; FileSystem
$Root 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Root', @ScriptDir)
$Temp 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Temp', @TempDir & '\' & $ScriptName)
$Cache 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Cache', @TempDir & '\' & $ScriptName & '\Cache')
$Home 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Home', '.\' & $UserName)
$Bin 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Bin', '.\Bin')
$Lib 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Lib', '.\Lib')
$Doc 		= _IniReadPlus($GlobalConfig, 'FileSystem', 'Doc', '.\Documents')
$Backup 	= _IniReadPlus($GlobalConfig, 'FileSystem', 'Backup', '.\Backups')
$Download 	= _IniReadPlus($GlobalConfig, 'FileSystem', 'Download', '.\Downloads')

; ------------------------------------------------------------------------------
; User Preferences
; ------------------------------------------------------------------------------
If FileExists($GlobalConfig) Then
	
	; Setup
	$UserName 	= _IniReadPlus($ScriptIni, 'Setup', 'UserName', $UserName)
	$Lang 		= _IniReadPlus($ScriptIni, 'Setup', 'Lang', $Lang)
	
	; FileSystem
	$Root		= _IniReadPlus($ScriptIni, 'FileSystem', 'Root', $Root)
	$Temp 		= _IniReadPlus($ScriptIni, 'FileSystem', 'Temp', $Temp)
	$Cache 		= _IniReadPlus($ScriptIni, 'FileSystem', 'Cache', $Cache)
	$Home 		= _IniReadPlus($ScriptIni, 'FileSystem', 'Home', $Home)
	$Bin 		= _IniReadPlus($ScriptIni, 'FileSystem', 'Bin', $Bin)
	$Lib 		= _IniReadPlus($ScriptIni, 'FileSystem', 'Lib', $Lib)
	$Doc 		= _IniReadPlus($ScriptIni, 'FileSystem', 'Doc', $Doc)
	$Backup 	= _IniReadPlus($ScriptIni, 'FileSystem', 'Backup', $Backup)
	$Download 	= _IniReadPlus($ScriptIni, 'FileSystem', 'Download', $Download)
EndIf

$Root 		= _FullPath ($Root)
$Temp 		= _FullPath ($Temp, $Root)
$Cache 		= _FullPath ($Cache, $Root)
$Home 		= _FullPath ($Home, $Root)
$Bin 		= _FullPath ($Bin, $Root)
$Lib 		= _FullPath ($Lib, $Root)
$Doc 		= _FullPath ($Doc, $Root)
$Backup 	= _FullPath ($Backup, $Root)
$Download 	= _FullPath ($Download, $Root)

$Drive = StringLeft($Root, 2)

; Create working dir
DirCreate($Root)
DirCreate($Temp)

; Install default files
$DefaultFile = $Temp & '\x-default'
FileInstall('x-default', $DefaultFile)

; ------------------------------------------------------------------------------
; FileToRun
; ------------------------------------------------------------------------------
$WorkingDir = _FullPath (IniRead($ScriptIni, 'FileToRun', 'WorkingDir', ''), $Root)
$PathToExe = _FullPath  (IniRead($ScriptIni, 'FileToRun', 'PathToExe', ''), $Root)

; Search PathToExe
If Not StringInStr($PathToExe, '\') Then
	If FileExists($WorkingDir & '\' & $PathToExe) Then
		$PathToExe = $WorkingDir & '\' & $PathToExe
	ElseIf FileExists(@ScriptDir & '\' & $PathToExe) Then
		$PathToExe = @ScriptDir & '\' & $PathToExe
	ElseIf FileExists(@WindowsDir & '\' & $PathToExe) Then
		$PathToExe = @WindowsDir & '\' & $PathToExe
	ElseIf FileExists(@SystemDir & '\' & $PathToExe) Then
		$PathToExe = @SystemDir & '\' & $PathToExe
	EndIf
EndIf

; Exe's details
$ExeDir = _FileInfo ($PathToExe, 0)
$ExeName = _FileInfo ($PathToExe, 1)

; ------------------------------------------------------------------------------
; Options
; ------------------------------------------------------------------------------

; Defaults
$DeleteTemp = 'true'
$FixAppData = 'false'
$MultipleIstances = 'true'
$RunWait = 'true'
$ShowSplash = 'true'
$WriteLog = 'false'

; Get options from config file
$_options = IniReadSection($ScriptIni, 'Options')
If Not @error Then
	For $o = 1 To $_options[0][0]
		If $_options[$o][1] = 'true' Or $_options[$o][1] = 'false' Then
			Select
				Case $_options[$o][0] = 'DeleteTemp'
					$DeleteTemp = $_options[$o][1]
				Case $_options[$o][0] = 'FixAppData'
					$FixAppData = $_options[$o][1]
				Case $_options[$o][0] = 'MultipleIstances'
					$MultipleIstances = $_options[$o][1]
				Case $_options[$o][0] = 'RunWait'
					$RunWait = $_options[$o][1]
				Case $_options[$o][0] = 'ShowSplash'
					$ShowSplash = $_options[$o][1]
				Case $_options[$o][0] = 'WriteLog'
					$WriteLog = $_options[$o][1]
			EndSelect
		EndIf
	Next
EndIf

; ------------------------------------------------------------------------------
; Log
; ------------------------------------------------------------------------------
$Log = @ScriptDir & '\' & $ScriptName & '.log'

If $WriteLog = 'true' Then
	IniWrite($Log, 'Setup', 'AppName', $AppName)
	IniWrite($Log, 'Setup', 'AppVer', $AppVer)
	IniWrite($Log, 'Setup', 'UserName', $UserName)
	IniWrite($Log, 'Setup', 'ProfileName', $Profile)
	IniWrite($Log, 'Setup', 'ScriptName', $ScriptName)
	IniWrite($Log, 'Setup', 'Lang', $Lang)
	IniWrite($Log, 'FileSystem', 'Root', $Root)
	IniWrite($Log, 'FileSystem', 'Temp', $Temp)
	IniWrite($Log, 'FileSystem', 'Cache', $Cache)
	IniWrite($Log, 'FileSystem', 'Home', $Home)
	IniWrite($Log, 'FileSystem', 'Bin', $Bin)
	IniWrite($Log, 'FileSystem', 'Lib', $Lib)
	IniWrite($Log, 'FileSystem', 'Doc', $Doc)
	IniWrite($Log, 'FileSystem', 'Backup', $Backup)
	IniWrite($Log, 'FileSystem', 'Download', $Download)
	IniWrite($Log, 'FileToRun', 'WorkingDir', $WorkingDir)
	IniWrite($Log, 'FileToRun', 'PathToExe', $PathToExe)
	IniWrite($Log, 'FileToRun', 'ExeDir', $ExeDir)
	IniWrite($Log, 'FileToRun', 'ExeName', $ExeName)
	IniWrite($Log, 'Environment', 'CD', $Root)
	IniWrite($Log, 'Environment', 'DRIVE', $Drive)
	IniWrite($Log, 'Environment', 'TEMP', $Temp)
	IniWrite($Log, 'Environment', 'LANG', $Lang)
EndIf

; Temp log
$TempLog = $Temp & '\' & $ScriptName & '.log'
$IsRunning = IniRead($TempLog, 'Status', 'IsRunning', '')

; Warning messages
Select
	Case $Lang = 'it'
		$CheckIni = 'Si prega di controllare le opzioni in ' & '"' & $ScriptIni & '".'
		$FileNotFound = 'ERRORE! "' & $PathToExe & '"' & ' non trovato!'
		$AlreadyRunning = '"' & $AppName & '"  gi in esecuzione.'
	Case Else
		$CheckIni = 'Please check options in ' & '"' & $ScriptIni & '".'
		$FileNotFound = 'ERROR! "' & $PathToExe & '"' & ' not found!'
		$AlreadyRunning = '"' & $AppName & '" is already running.'
EndSelect

; Check File To Run
If Not FileExists($PathToExe) Then
	MsgBox(48, $ScriptName, $FileNotFound)
	MsgBox(48, $ScriptName, $CheckIni)
	RunWait(@ComSpec & ' /c "' & $ScriptIni & '"', '', @SW_HIDE)
	Exit
EndIf

; Multiple istances	
If $MultipleIstances = 'false' Then
	AutoItSetOption('WinTitleMatchMode', 2)
	$WinGetProcess = IniRead($ScriptIni, 'FileToRun', 'WinGetProcess', '')
	Select
		Case $WinGetProcess = ''
			If ProcessExists($ExeName) Then 
				MsgBox(48, $ScriptName, $AlreadyRunning)
				Exit
			EndIf
		Case Else
			If WinGetProcess($WinGetProcess) <> - 1 Then 
				MsgBox(48, $ScriptName, $AlreadyRunning)
				Exit
			EndIf
	EndSelect
EndIf

; ------------------------------------------------------------------------------
; SplashScreen
; ------------------------------------------------------------------------------

; Image
$Image = _IniReadPlus($ScriptIni, 'SplashScreen', 'Image', '')
$Image = _FullPath ($Image, $Root)

; Title
$Title = _IniReadPlus($ScriptIni, 'SplashScreen', 'Title', '')

; Timeout
$TimeOut = _IniReadPlus($ScriptIni, 'SplashScreen', 'TimeOut ', '')
Select
	Case StringIsDigit($TimeOut)
	Case Else
		$TimeOut = '1500'
EndSelect

; ------------------------------------------------------------------------------
; Get Environmental Variables
; ------------------------------------------------------------------------------
$_environment = IniReadSection($ScriptIni, 'Environment')
If Not @error Then
	For $e = 1 To $_environment[0][0]
		Select
			Case $_environment[$e][0] = 'USERPROFILE'
				$UserProfile = _FullPath ($_environment[$e][1], $Root)
				EnvSet('USERPROFILE', $UserProfile)
				If $WriteLog = 'true' Then IniWrite($Log, 'Environment', 'USERPROFILE', $UserProfile)
				If $FixAppData = 'true' Then
					
					; Fix AppData
					$AppData = _DirName (@AppDataDir)
					_FixUserProfile ($UserProfile, $AppData, 'AppData')
					EnvSet('APPDATA', $UserProfile & '\' & $AppData)
					If $WriteLog = 'true' Then IniWrite($Log, 'Environment', 'APPDATA', $AppData)
					
					; Fix Desktop
					$Desktop = _DirName (@DesktopDir)
					_FixUserProfile ($UserProfile, $Desktop, 'Desktop')
					DirCreate($UserProfile & '\' & $Desktop)
					
					; Fix Documents
					$Documents = _DirName (@MyDocumentsDir)
					_FixUserProfile ($UserProfile, $Documents, 'Documents')
					DirCreate($UserProfile & '\' & $Documents)
					
					; Fix Favorites
					$Favorites = _DirName (@FavoritesDir)
					_FixUserProfile ($UserProfile, $Favorites, 'Favorites')
					
				EndIf	
			Case $_environment[$e][0] = 'PATH'
				_SetPath ($_environment[$e][1], $WriteLog, $Log)
			Case Else
				_SetEnv($_environment[$e][0], $_environment[$e][1], $WriteLog, $Log)
		EndSelect
	Next
EndIf

; ------------------------------------------------------------------------------
; Functions
; ------------------------------------------------------------------------------
$_functions = IniReadSection($ScriptIni, 'Functions')
If Not @error Then
	For $f = 1 To $_functions[0][0]
		Select
			Case $_functions[$f][1] = ''
			Case $_functions[$f][0] = 'DirCopy'
				_DirCopy ($_functions[$f][1])
			Case $_functions[$f][0] = 'DirCreate'
				_DirCreate ($_functions[$f][1])
			Case $_functions[$f][0] = 'DirMove'
				_DirMove ($_functions[$f][1])
			Case $_functions[$f][0] = 'DirRemove'
				DirRemove(_FullPath ($_functions[$f][1]), 1)
			Case $_functions[$f][0] = 'FileCopy'
				_FileCopy ($_functions[$f][1])
			Case $_functions[$f][0] = 'FileCreate'
				_FileCreate($_functions[$f][1])
			Case $_functions[$f][0] = 'FileDelete'
				_FileDelete ($_functions[$f][1])
			Case $_functions[$f][0] = 'FileMove'
				_FileMove ($_functions[$f][1])
		EndSelect
	Next
EndIf

; ------------------------------------------------------------------------------
; Sections
; ------------------------------------------------------------------------------
$_sections = IniReadSectionNames($ScriptIni)
For $i = 1 To $_sections[0]
	$_file = StringSplit($_sections[$i], '=')
	If Not @error Then
		AutoItSetOption('ExpandEnvStrings', 0)
		AutoItSetOption('ExpandVarStrings', 0)
		$_values = IniReadSection($ScriptIni, $_sections[$i])
		AutoItSetOption('ExpandEnvStrings', 1)
		AutoItSetOption('ExpandVarStrings', 1)
		Select
			Case $_file[1] = 'StringReplace'
				If Not @error Then
					Local $FileToRewrite, $_delimiters
					$FileToRewrite = _FullPath ($_file[2], $Root)
					If FileExists($FileToRewrite) Then
						For $k = 1 To $_values[0][0]
							$_delimiters = StringSplit($_values[$k][0], '|')
							_StringReplace ($FileToRewrite, $_delimiters[1], $_delimiters[2], _FullPathPlus ($_values[$k][1]))
						Next
					EndIf
				EndIf
			Case $_file[1] = 'WriteToFile'
				If Not @error Then
					Local $FileToWrite
					$FileToWrite = _FullPath ($_file[2], $Root)
					For $k = 1 To $_values[0][0]
						_WriteToFile ($FileToWrite, $_values[$k][0], $_values[$k][1])
					Next
				EndIf
			Case $_file[1] = 'WriteToIni'
				If Not @error Then
					Local $IniFile, $_stw
					$IniFile = _FullPath ($_file[2], $Root)
					For $k = 1 To $_values[0][0]
						$_stw = StringSplit($_values[$k][0], '|')
						IniWrite($IniFile, $_stw[1], $_stw[2], _FullPathPlus ($_values[$k][1]))
					Next
				EndIf
			Case $_file[1] = 'WriteToPref'
				If Not @error Then
					Local $PrefsFile, $Format, $_begin, $_mid, $_end, $Begin, $Mid, $End
					$PrefsFile = _FullPath ($_file[2], $Root)
					$Format = $_values[1][1]
					$_begin = StringSplit($format, '[PREF]', 1)
					$_mid = StringSplit($_begin[2], '[VALUE]', 1)
					$_end = StringSplit($format, '[VALUE]', 1)
					$Begin = $_begin[1]
					$Mid = $_mid[1]
					If $_end[0] = 2 Then
						$End = $_end[2]
					Else
						$End = ''
					EndIf
					For $k = 2 To $_values[0][0]
						_WriteToPref ($PrefsFile, $Begin, $Mid, $End, $_values[$k][0], _FullPathPlus($_values[$k][1]))
					Next
				EndIf
			Case $_file[1] = 'WriteToReg'
				If Not @error Then
					Local $RegFile, $MainKey, $_stw 
					$RegFile = _FullPath ($_file[2], $Root)
					$MainKey = $_values[1][1]
					_WriteToReg ($RegFile, $MainKey, '', '', '')
					For $k = 2 To $_values[0][0]
						$_stw = StringSplit($_values[$k][0], '|')
						Select
							Case $_stw[0] = 1
								_WriteToReg ($RegFile, $MainKey, '', $_stw[1], $_values[$k][1])
							Case $_stw[0] = 2
								_WriteToReg ($RegFile, $MainKey, $_stw[1], $_stw[2], $_values[$k][1])
						EndSelect
					Next
				EndIf
		EndSelect
	EndIf
Next

; ------------------------------------------------------------------------------
; RunBefore
; ------------------------------------------------------------------------------
$_runbefore = IniReadSection($ScriptIni, 'RunBefore')
If Not @error Then
	For $rb = 1 To $_runbefore[0][0]
		Select
			Case $_runbefore[$rb][1] = ''
			Case $_runbefore[$rb][0] = 'FixDriveLetter'
				_FixDriveLetter ($_runbefore[$rb][1], $Root)
			Case $_runbefore[$rb][0] = 'Regedit'
				If $IsRunning <> 'true' Then _RegFileInstall ($_runbefore[$rb][1], $Temp & '\Regedit\backup' & $rb)
			Case $_runbefore[$rb][0] = 'RunFile'
				_RunWait ($_runbefore[$rb][1], $Root)
		EndSelect
	Next
EndIf

; ------------------------------------------------------------------------------
; FileToRun
; ------------------------------------------------------------------------------

; Parameters
$Parameters = IniRead($ScriptIni, 'FileToRun', 'Parameters', '')
If $Parameters <> '' Then
	Select
		Case StringLeft($Parameters, 1) = '-' Or StringLeft($Parameters, 1) = '/'
			$Parameters = ' ' & $Parameters
		Case Else
			$Parameters = ' "' & $Parameters & '"'
	EndSelect
EndIf

; CommandLine
If $CmdLine[0] >= $Cmd1 Then
	$Cmd = $CmdLine[$Cmd1]
	For $i = $Cmd1 + 1 To $CmdLine[0]
		$Cmd = $Cmd & ' ' & $CmdLine[$i]
	Next
	Select
		Case StringLeft($Cmd, 1) = '-' Or StringLeft($Cmd, 1) = '/'
			$Cmd = ' ' & $Cmd
		Case Else
			$Cmd = ' "' & $Cmd & '"'
	EndSelect
Else
	$Cmd = ''
EndIf

; ChangeDir
$StringToExe = '"' & $PathToExe & '"' & $Parameters & $Cmd
If $WorkingDir <> '' And FileExists($WorkingDir) Then
	FileChangeDir($WorkingDir)
Else
	FileChangeDir($ExeDir)
EndIf

; Write Log
If $WriteLog = 'true' Then
	IniWrite($Log, 'FileToRun', 'StringToExe', $StringToExe)
EndIf
IniWrite($TempLog, 'Status', 'IsRunning', 'true')

; RunFile
Run($StringToExe)

; ShowSplash
If $ShowSplash = 'true' And $IsRunning <> 'true' Then 
	_SplashScreen ($Title, $Image, $TimeOut, $Temp, $Root)
EndIf

If $RunWait = 'true' And $IsRunning <> 'true' Then 
	ProcessWaitClose($ExeName)
Else
	IniWrite($TempLog, 'Status', 'IsRunning', 'false')
	Exit
EndIf

; ------------------------------------------------------------------------------
; RunAfter
; ------------------------------------------------------------------------------
$_runafter = IniReadSection($ScriptIni, 'RunAfter')
If Not @error Then
	For $ra = 1 To $_runafter[0][0]
		Select
			Case $_runafter[$ra][0] = 'DirCopy'
				_DirCopy ($_runafter[$ra][1])
			Case $_runafter[$ra][0] = 'DirMove'
				_DirMove ($_runafter[$ra][1])
			Case $_runafter[$ra][0] = 'DirRemove'
				DirRemove(_FullPath ($_runafter[$ra][1]), 1)
			Case $_runafter[$ra][0] = 'FileCopy'
				_FileCopy ($_runafter[$ra][1])
			Case $_runafter[$ra][0] = 'FileDelete'
				_FileDelete ($_runafter[$ra][1])
			Case $_runafter[$ra][0] = 'FileMove'
				_FileMove ($_runafter[$ra][1])
			Case $_runafter[$ra][0] = 'RunFile'
				_RunWait ($_runafter[$ra][1], $Root)
		EndSelect
	Next
EndIf

; Restore Regkeys
$_restorekeys = IniReadSection($ScriptIni, 'RunBefore')
If Not @error Then
	For $rk = 1 To $_restorekeys[0][0]
		Select
			Case $_restorekeys[$rk][0] = 'Regedit'
				_RegFileRestore ($_restorekeys[$rk][1], $Temp & '\Regedit\backup' & $rk)
		EndSelect
	Next
EndIf

; Exit
IniWrite($TempLog, 'Status', 'IsRunning', 'false')
If $DeleteTemp = 'true' Then DirRemove($Temp, 1)
;
; ==> End of X-Launcher's code
; ------------------------------------------------------------------------------
#include 'x-udf.au3'
